unit FromThreadProblem2_;

interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, VidGrab, StdCtrls, ExtCtrls;

type

  TVgThreadAction = (ta_Init, ta_Destroy, ta_StartPreview, ta_StopPreview, ta_CaptureFrame);


  TCameraThread = Class(TThread)
  Private
      FDoAction: THandle;
      FVgThreadAction: TVgThreadAction;
      FIPCamera: TVideoGrabber;
  Public
      constructor Create(CreateSuspended: Boolean);
      procedure SetVgParent;
      procedure SetVgNone;
      procedure CaptureFrame;
      procedure VideoGrabberFrameProgress2(Sender: TObject; FrameInfo: pFrameInfo);
      function Initiate (VgThreadAction: TVgThreadAction): Boolean;
      destructor Destroy; override;
      Procedure Execute; Override;
      procedure OnThreadSync (Sender: TObject; ThreadSyncPoint: TThreadSyncPoint);
  End;

  TForm3 = class(TForm)
    Button1: TButton;
    Panel1: TPanel;
    Button2: TButton;
    mmoLog: TMemo;
    edtFrameCount: TEdit;
    Button3: TButton;
    Button5: TButton;
    IPCameraURL: TEdit;
    Label1: TLabel;
    Image1: TImage;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button5Click(Sender: TObject);
  private
     CameraThread: TCameraThread;
  public
  end;

var
  Form3: TForm3;

implementation

{$R *.dfm}

constructor TCameraThread.Create(CreateSuspended: Boolean);
begin
   inherited Create (CreateSuspended);
   FVgThreadAction := ta_Init;
   FDoAction := CreateEvent (nil, true, true, nil);
   Priority := tpLower;
   FreeOnTerminate := true;
   Resume;
end;

destructor TCameraThread.Destroy;
begin
   CloseHandle (FDoAction);
end;

function TCameraThread.Initiate (VgThreadAction: TVgThreadAction): Boolean;
begin
   Result := false;
   if WaitForSingleObject (FDoAction, 0) <> WAIT_TIMEOUT then Exit;
   FVgThreadAction := VgThreadAction;
   SetEvent (FDoAction);
   if VgThreadAction = ta_destroy then begin
       WaitFor;
   end;
end;

procedure TCameraThread.VideoGrabberFrameProgress2(Sender: TObject; FrameInfo: pFrameInfo);
var
   DVTimeCode: String;
   DVDateTime: String;
begin
   with FrameInfo^ do begin
      if CurrentState = cs_Playback then begin // no dropped frames in playback mode
         Form3.edtFrameCount.Text := Format ('frame: %d time:%.2d:%.2d:%.2d %.2d', [FrameNumber, FrameTime_Hour, FrameTime_Min, FrameTime_Sec, FrameTime_Hs]);
      end
      else begin
         Form3.edtFrameCount.Text := Format ('frame: %d dropped: %d time:%.2d:%.2d:%.2d %.2d', [FrameNumber, DroppedFrameCount, FrameTime_Hour, FrameTime_Min, FrameTime_Sec, FrameTime_Hs]);
      end;

      Form3.mmoLog.Lines.Add (Form3.edtFrameCount.Text);
   end;
end;

procedure TForm3.Button1Click(Sender: TObject);
begin
   if assigned (CameraThread) then begin
      CameraThread.Initiate (ta_StartPreview);
   end;
 end;

procedure TCameraThread.SetVgParent;
begin
       FIPCamera.Parent := FOrm3.Panel1;
end;

procedure TCameraThread.CaptureFrame;
var
   Bitmap: TBitmap;
begin
   Bitmap := FIPCamera.GetLastFrameAsTBitmap (0, true, 0, 0, 0, 0, 0, 0, 0);
   if assigned (Bitmap) then begin
      Form3.Image1.Picture.Assign (Bitmap);
      Bitmap.Free;
   end;
end;

procedure TCameraThread.SetVgNone;
begin
       FIPCamera.Parent := nil;
end;

procedure TCameraThread.OnThreadSync (Sender: TObject; ThreadSyncPoint: TThreadSyncPoint);
begin
   case ThreadSyncPoint of
      tsp_Starting_CanSetParent:
      begin
         Synchronize (SetVgParent);
      end;
      tsp_Started_CanSetParent:
      begin
         Synchronize (SetVgParent);
      end;
      tsp_Stop_UnSetParent:
      begin
         Synchronize (SetVgNone);
      end;
   end;
end;

procedure TCameraThread.Execute;
var
   Done: Boolean;
begin
 Done := false;
 while (not Done) do begin
   case FVgThreadAction of
      ta_Init: begin
         if not assigned (FIPCamera) then begin
            FIPCamera := TVideoGrabber.Create(nil);  // the component must be created and assigned the Parent from the thread
            FIPCamera.EnableThreadMode;
         end;
      end;
      ta_StartPreview: begin
         if assigned (FIPCamera) then begin
            FIPCamera.VideoSource:=vs_IPCamera;
            FIPCamera.IPCameraURL := Form3.IPCameraURL.Text;
            //FIPCamera.SetAuthentication (at_IPCamera, 'root', 'admin'); // if needed...
            FIPCamera.OnFrameProgress2 := VideoGrabberFrameProgress2;
            FIPCamera.OnThreadSync := OnThreadSync;
            if FIPCamera.StartPreview then begin
               Form3.mmoLog.Lines.Add ('previewing...');
            end;
         end;
      end;
      ta_StopPreview: begin
         if assigned (FIPCamera) then begin
            FIPCamera.StopPreview;
            Form3.mmoLog.Lines.Add ('stopped.');
         end;
      end;

      ta_CaptureFrame: begin
         if assigned (FIPCamera) then begin
            Synchronize (CaptureFrame);
         end;
      end;

      ta_Destroy: begin
         if assigned (FIPCamera) then begin
            FIPCamera.Free;
            FIPCamera := nil;
            Done := true;
         end;
         Break;
      end;
   end;
   ResetEvent (FDoAction);
   WaitForSingleObject (FDoAction, INFINITE);
 end;
end;

procedure TForm3.Button2Click(Sender: TObject);
begin
   if assigned (CameraThread) then begin
      CameraThread.Initiate (ta_StopPreview);
   end;
end;

procedure TForm3.Button3Click(Sender: TObject);
begin
   if assigned (CameraThread) then begin
      CameraThread.Initiate (ta_Destroy);
      CameraThread := nil;
      mmoLog.Lines.Add ('destroyed.');
   end
   else begin
      mmoLog.Lines.Add ('already destroyed!');
   end;
end;

procedure TForm3.Button4Click(Sender: TObject);
begin
   if not assigned (CameraThread) then begin
      CameraThread := TCameraThread.Create (true);
      mmoLog.Lines.Add ('created.');
   end
   else begin
      mmoLog.Lines.Add ('already created!');
   end;
end;

procedure TForm3.Button5Click(Sender: TObject);
begin
   if assigned (CameraThread) then begin
      CameraThread.Initiate (ta_CaptureFrame);
   end;
end;

procedure TForm3.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   if assigned (CameraThread) then begin
      CameraThread.Initiate (ta_Destroy);
   end;
end;

end.

